// Libraries
import { createSelector } from '@ngrx/store';

// Models
import { GlobalState } from '@app/core/state';
import { Year } from '@app/core/models/year';
import { Note } from '@app/core/models/note';
import { Course } from '@app/core/models/course';
import { DegreePlannerState } from './state';

export const getDegreePlannerState = ({ degreePlanner }: GlobalState) => {
  return degreePlanner;
};

export const getSavedForLaterCourses = createSelector(
  getDegreePlannerState,
  state => state.savedForLaterCourses,
);

export const firstActiveTermCode = createSelector(
  getDegreePlannerState,
  state => {
    if (state.activeTermCodes.length > 0) {
      return state.activeTermCodes[0];
    } else {
      return undefined;
    }
  },
);

export const getAllDegreePlans = createSelector(
  getDegreePlannerState,
  state => state.allDegreePlans,
);

export const getVisibleRoadmapId = createSelector(
  getDegreePlannerState,
  state => {
    return state.visibleDegreePlan
      ? state.visibleDegreePlan.roadmapId
      : undefined;
  },
);

export const getVisibleDegreePlan = createSelector(
  getDegreePlannerState,
  state => state.visibleDegreePlan,
);

export const getAllVisibleTerms = createSelector(
  getDegreePlannerState,
  state => state.visibleTerms,
);

export const getAllVisibleTermsByYear = createSelector(
  getDegreePlannerState,
  state => {
    const unqiueYears = state.visibleTerms
      .map(term => term.termCode.slice(0, 3))
      .filter(year => year.match(/^\d{3}/))
      .filter((year, index, self) => self.indexOf(year) === index)
      .sort();

    const allNotes = state.visibleTerms
      .map(term => term.note)
      .filter((note): note is Note => note !== undefined);

    const allCourses = state.visibleTerms
      .map(term => term.courses)
      .reduce((flat, nested) => flat.concat(nested), [])
      .map(course => {
        course.subject = state.subjects[course.subjectCode];
        return course;
      });

    return unqiueYears.map<Year>(year => {
      const century = year[0] === '0' ? 0 : 1;
      const twoDigitYearCode = parseInt(year.substr(1, 2), 10);
      return {
        century,
        twoDigitYearCode,
        fall: getTermForCode(`${year}2`, allCourses, allNotes),
        spring: getTermForCode(`${year}4`, allCourses, allNotes),
        summer: getTermForCode(`${year}6`, allCourses, allNotes),
      };
    });
  },
);

export const getDropZones = createSelector(
  getDegreePlannerState,
  (state: DegreePlannerState) => {
    return [
      'saved-courses',
      ...state.activeTermCodes.map(termCode => {
        return `term-${termCode}`;
      }),
    ];
  },
);

export const isActiveTerm = (termCode: string) =>
  createSelector(
    getDegreePlannerState,
    (state: DegreePlannerState) => {
      return state.activeTermCodes.includes(termCode);
    },
  );

const getTermForCode = (termCode: string, courses: Course[], notes: Note[]) => {
  return {
    termCode,
    note: notes.find(note => note.termCode === termCode),
    courses: courses.filter(course => course.termCode === termCode),
  };
};
